# To run this you need to first install pygame.  
# It should work regardless of platform, but python 2.x is required
# Author: pflarr
# License: Free to steal (but it's known who made this)
import pygame
from pygame.locals import *
import pickle
import sys
import time
import os

if len(sys.argv) != 2:
    print >> sys.stderr, """usage: python editor.py <savefile>
savefile - the file to save the image state in.  This is loaded if it
           already exists.  This is NOT what should be uploaded to the device.
The output is a sequence of raw bytes meant to be fed directly to the badge
over serial, after entering USB mode using #.

sh$ python editor.py <savefile> > rawfile

The image will stay around for a while, but button presses or time will erase
it.  The best way  to do this would be to put the image directly on the device
via reprogramming it."""
    exit(0)

savefile = sys.argv[1]

if os.path.exists(savefile):
    bytes = pickle.load(open(savefile))
else:
    bytes = [[0xff for i in range(128)] for j in range(4)]

PSIZE = 7

pygame.init()

screen = pygame.display.set_mode((128*PSIZE, 32*PSIZE))
quit = False

WHITE = (220,220,220)
BLACK = (50,50,50)

def getpos(mousepos):
    x = mousepos[1]/PSIZE
    y = mousepos[0]/PSIZE
    return x,y

def draw_byte(screen, x, y, byte):
    for bit in range(8):
        if byte & (0x01 << bit):
            color = WHITE 
        else:
            color = BLACK 
        rect = (y*PSIZE, x*8*PSIZE + bit*PSIZE, PSIZE, PSIZE)
        screen.fill(color, rect)

for x in range(len(bytes)):
    for y in range(len((bytes[x]))):
            draw_byte(screen, x, y, bytes[x][y]) 
pygame.display.flip()

EVENTLIST = [MOUSEBUTTONDOWN, MOUSEBUTTONUP, QUIT]
pygame.event.set_allowed(EVENTLIST)
dstate = None

while (not quit):
    if not pygame.event.peek():
        time.sleep(.1)
        continue
    events = pygame.event.get()
    for event in events:
        if event.type == MOUSEBUTTONDOWN and event.button == 1:
            pos = getpos(event.pos)
            byte = bytes[pos[0]/8][pos[1]]
            mask = 0x01 << (pos[0] % 8)
            dstate = byte & mask > 0
            if dstate:
                byte = byte & (mask ^ 0xff)
            else:
                byte = byte | (mask)
            draw_byte(screen, pos[0]/8, pos[1], byte) 
            bytes[pos[0]/8][pos[1]] = byte
        elif event.type == MOUSEMOTION and dstate is not None:
            pos = getpos(event.pos)
            byte = bytes[pos[0]/8][pos[1]]
            mask = 0x01 << (pos[0] % 8)
            if dstate:
                byte = byte & (mask ^ 0xff)
            else:
                byte = byte | (mask)
            draw_byte(screen, pos[0]/8, pos[1], byte) 
            bytes[pos[0]/8][pos[1]] = byte
 
        elif event.type == MOUSEBUTTONUP and event.button == 1:
            dstate = None
        elif event.type == QUIT:
            quit = True
    pygame.display.flip()

file = open(savefile, 'w')
pickle.dump(bytes, file)

ob = [0x43]
for x in range(len(bytes)):
    for y in range(len(bytes[x])):
        if bytes[x][y] == 0xff:
            continue
        ob.append(0x4C)
        pos = 0x80*x + y
        ob.append((pos & 0xff00) >> 8)
        ob.append(pos & 0xff)
        ob.append(bytes[x][y])
ob.append(0x55)

ob = ['%c' % v for v in ob]
print ''.join(ob)
